@@ -50,11 +50,9 @@ setMethod("dbExistsTable", c("spark_connection", "character"), function(conn, na
 
 setMethod("dbRemoveTable", c("spark_connection", "character"),
   function(conn, name) {
-    hive <- hive_context(conn)
-    if (is_spark_v2(conn)) {
-      hive <- invoke(hive, "sqlContext")
-    }
-    invoke(hive, "dropTempTable", name)
+    dbi_ensure_no_backtick(name)
+
+    dbSendQuery(conn, paste0("DROP TABLE `", name, "`"))
     invisible(TRUE)
   }
 )
