@@ -25,6 +25,13 @@ livy_validate_http_response <- function(message, req) {
   }
 }
 
+livy_available_jars <- function() {
+  system.file("java", package = "sparklyr") %>%
+    dir(pattern = "sparklyr") %>%
+    gsub("^sparklyr-|-.*\\.jar", "", .) %>%
+    numeric_version()
+}
+
 #' Create a Spark Configuration for Livy
 #'
 #' @export
@@ -41,8 +48,8 @@ livy_validate_http_response <- function(message, req) {
 #'
 #' @details
 #'
-#' Extends a Spark \code{"spark_config"} configuration with settings
-#' for Livy. For instance, \code{"username"} and \code{"password"}
+#' Extends a Spark \code{spark_config()} configuration with settings
+#' for Livy. For instance, \code{username} and \code{password}
 #' define the basic authentication settings for a Livy session.
 #'
 #' The default value of \code{"custom_headers"} is set to \code{list("X-Requested-By" = "sparklyr")}
@@ -70,8 +77,14 @@ livy_validate_http_response <- function(message, req) {
 #' \code{config = spark_config(spark.yarn.queue = "my_queue")}).
 #'
 #' @return Named list with configuration data
-livy_config <- function(config = spark_config(), username = NULL, password = NULL, negotiate = FALSE,
-                        custom_headers = list("X-Requested-By" = "sparklyr"), ...) {
+livy_config <- function(config = spark_config(),
+                        username = NULL,
+                        password = NULL,
+                        negotiate = FALSE,
+                        custom_headers = list("X-Requested-By" = "sparklyr"),
+                        ...) {
+  additional_params <- list(...)
+
   if (negotiate) {
     config[["sparklyr.livy.auth"]] <- httr::authenticate("", "", type = "gssnegotiate")
   } else if (!is.null(username) || !is.null(password)) {
@@ -86,10 +99,20 @@ livy_config <- function(config = spark_config(), username = NULL, password = NUL
   }
 
   #Params need to be restrictued or livy will complain about unknown parameters
-  allowed_params <- c("proxy_user", "jars", "py_files", "files", "driver_memory", "driver_cores", "executor_memory",
-                      "executor_cores", "num_executors", "archives", "queue", "name", "heartbeat_timeout")
-
-  additional_params <- list(...)
+  allowed_params <- c("proxy_user",
+                      "jars",
+                      "py_files",
+                      "files",
+                      "driver_memory",
+                      "driver_cores",
+                      "executor_memory",
+                      "executor_cores",
+                      "num_executors",
+                      "archives",
+                      "queue",
+                      "name",
+                      "heartbeat_timeout",
+                      "conf")
 
   if (length(additional_params) > 0) {
     valid_params <- names(additional_params) %in% allowed_params
@@ -114,7 +137,8 @@ livy_config <- function(config = spark_config(), username = NULL, password = NUL
       archives = "archives",
       queue = "queue",
       name = "name",
-      heartbeat_timeout = "heartbeatTimeoutInSecond"
+      heartbeat_timeout = "heartbeatTimeoutInSecond",
+      conf = "conf"
     )
 
     for (l in names(additional_params)){
@@ -165,7 +189,7 @@ livy_config_get_prefix <- function(master, config, prefix, not_prefix) {
   ), prefix, not_prefix)
 
   params <- lapply(params, function(param) {
-    unbox(param)
+    if (length(param) == 1) unbox(param) else param
   })
 
   if (length(params) == 0) {
@@ -278,33 +302,31 @@ livy_serialized_chunks <- function(serialized, n) {
 
 livy_statement_compose <- function(sc, static, class, method, ...) {
   serialized <- livy_invoke_serialize(sc = sc, static = static, object = class, method = method, ...)
-  chunks <- livy_serialized_chunks(serialized, 1000)
+  chunks <- livy_serialized_chunks(serialized, 10000)
 
   chunk_vars <- list()
   last_var <- NULL
-  for (i in 1:length(chunks)) {
-    if (is.null(last_var)) {
-      var_name <- paste("\"", chunks[i], "\"", sep = "")
-    }
-    else {
-      if (length(chunk_vars) == 0) {
-        var_name <- livy_code_new_return_var(sc)
-        chunk_vars <- c(chunk_vars, paste("var ", var_name, " = ", last_var, sep = ""))
-        last_var <- var_name
-      }
+  var_name <- "sparklyr_return"
 
-      var_name <- livy_code_new_return_var(sc)
-      chunk_vars <- c(chunk_vars, paste("var ", var_name, " = ", last_var, " + \"", chunks[i], "\"", sep = ""))
+  if (length(chunks) == 1) {
+    last_var <- paste("\"", chunks[1], "\"", sep = "")
+  }
+  else {
+    last_var <- "builder.toString"
+    chunk_vars <- c(chunk_vars, "val builder = StringBuilder.newBuilder")
+    for (i in 1:length(chunks)) {
+      chunk_vars <- c(chunk_vars, paste("builder.append(\"", chunks[i], "\") == \"\"", sep = ""))
     }
-
-    last_var <- var_name
   }
 
   var_name <- livy_code_new_return_var(sc)
 
   invoke_var <- paste(
     "var ", var_name, " = ",
-    "LivyUtils.invokeFromBase64(",
+    paste0(
+      livy_map_class(sc, "sparklyr.LivyUtils.invokeFromBase64"),
+      "("
+    ),
     last_var,
     ")",
     sep = ""
@@ -531,6 +553,26 @@ livy_connection_not_used_warn <- function(value, default = NULL, name = deparse(
   }
 }
 
+livy_connection_jars <- function(config, version) {
+  livy_jars <- list()
+
+  if (!spark_config_value(config, "sparklyr.livy.sources", FALSE)) {
+    major_version <- gsub("\\.$", "", version)
+    previouis_versions <- Filter(function(maybe_version) maybe_version <= major_version, livy_available_jars())
+    target_version <- previouis_versions[length(previouis_versions)]
+
+    target_jar <- dir(system.file("java", package = "sparklyr"), pattern = paste0("sparklyr-", target_version))
+
+    livy_jars <- list(paste0(
+      "https://github.com/rstudio/sparklyr/blob/bugfix/livy-jars/inst/java/",
+      target_jar,
+      "?raw=true"
+    ))
+  }
+
+  livy_jars
+}
+
 livy_connection <- function(master,
                             config,
                             app_name,
@@ -539,7 +581,6 @@ livy_connection <- function(master,
                             extensions) {
 
   livy_connection_not_used_warn(app_name, "sparklyr")
-  livy_connection_not_used_warn(version)
   livy_connection_not_used_warn(hadoop_version)
   livy_connection_not_used_warn(extensions, registered_extensions())
 
@@ -552,6 +593,15 @@ livy_connection <- function(master,
 
   livy_validate_master(master, config)
 
+  if (!is.null(version) && !spark_config_value(config, "sparklyr.livy.sources", FALSE)) {
+    extensions <- spark_dependencies_from_extensions(version, extensions, config)
+
+    config$livy.jars <- as.character(c(livy_connection_jars(config, version), extensions$catalog_jars))
+
+    config[["sparklyr.livy.sources"]] <- FALSE
+    config[["spark.jars.packages"]] <- paste(c(config[["spark.jars.packages"]], extensions$packages), collapse = ",")
+  }
+
   session <- livy_create_session(master, config)
 
   sc <- new_livy_connection(list(
@@ -561,6 +611,7 @@ livy_connection <- function(master,
     app_name = app_name,
     config = config,
     state = new.env(),
+    extensions = extensions,
     # livy_connection
     sessionId = session$id,
     code = new.env(),
@@ -569,7 +620,7 @@ livy_connection <- function(master,
 
   sc$code$totalReturnVars <- 0
 
-  waitStartTimeout <- spark_config_value(config, "livy.session.start.timeout", 60)
+  waitStartTimeout <- spark_config_value(config, c("sparklyr.connect.timeout", "livy.session.start.timeout"), 60)
   waitStartReties <- waitStartTimeout * 10
   while (session$state == "starting" &&
          session$state != "dead" &&
@@ -648,8 +699,14 @@ spark_disconnect.livy_connection <- function(sc, ...) {
   }
 }
 
-livy_map_class <- function(class) {
-  gsub("sparklyr.", "", class)
+livy_map_class <- function(sc, class) {
+  if (spark_config_value(sc$config, "sparklyr.livy.sources", TRUE)) {
+    gsub("sparklyr.", "", class)
+  }
+  else {
+    # if sources are provided as a jar, sources contain spakrlyr as proper package
+    class
+  }
 }
 
 #' @export
@@ -671,14 +728,14 @@ invoke.livy_jobj <- function(jobj, method, ...) {
 
 #' @export
 invoke_static.livy_connection <- function(sc, class, method, ...) {
-  classMapped <- livy_map_class(class)
+  classMapped <- livy_map_class(sc, class)
 
   livy_invoke_statement_fetch(sc, TRUE, classMapped, method, ...)
 }
 
 #' @export
 invoke_new.livy_connection <- function(sc, class, ...) {
-  class <- livy_map_class(class)
+  class <- livy_map_class(sc, class)
 
   livy_invoke_statement_fetch(sc, TRUE, class, "<init>", ...)
 }
@@ -697,6 +754,9 @@ livy_load_scala_sources <- function(sc) {
     "serializer.scala",
     "stream.scala",
     "repartition.scala",
+    "arrowhelper.scala",
+    "arrowbatchstreamwriter.scala",
+    "arrowconverters.scala",
     "applyutils.scala",
     "classutils.scala",
     "fileutils.scala",
@@ -710,7 +770,6 @@ livy_load_scala_sources <- function(sc) {
     "workerapply.scala",
     "workerrdd.scala",
     "workerhelper.scala",
-    "workerutils.scala",
     "mlutils.scala",
     "mlutils2.scala",
     "bucketizerutils.scala",
@@ -760,17 +819,24 @@ livy_load_scala_sources <- function(sc) {
 #' @export
 initialize_connection.livy_connection <- function(sc) {
   tryCatch({
-    livy_load_scala_sources(sc)
 
-    session <- NULL
-    sc$state$spark_context <- tryCatch({
-      session <<- invoke_static(
+    if (spark_config_value(sc$config, "sparklyr.livy.sources", TRUE)) {
+      livy_load_scala_sources(sc)
+    }
+
+    session <- tryCatch({
+      invoke_static(
         sc,
         "org.apache.spark.sql.SparkSession",
         "builder"
       ) %>%
         invoke("getOrCreate")
+    },
+    error = function(e) {
+      NULL
+    })
 
+    sc$state$spark_context <- tryCatch({
       invoke(session, "sparkContext")
     },
     error = function(e) {
