@@ -26,6 +26,7 @@ ml_bisecting_kmeans <- function(x, formula = NULL, k = 4, max_iter = 20,
                                 features_col = "features", prediction_col = "prediction",
                                 uid = random_string("bisecting_bisecting_kmeans_"),
                                 ...) {
+  check_dots_used()
   UseMethod("ml_bisecting_kmeans")
 }
 
@@ -44,11 +45,12 @@ ml_bisecting_kmeans.spark_connection <- function(x, formula = NULL, k = 4, max_i
     prediction_col = prediction_col
   ) %>%
     c(rlang::dots_list(...)) %>%
-    ml_validator_bisecting_kmeans()
+    validator_ml_bisecting_kmeans()
 
-  jobj <- ml_new_clustering(
+  jobj <- spark_pipeline_stage(
     x, "org.apache.spark.ml.clustering.BisectingKMeans", uid,
-    .args[["features_col"]], .args[["k"]], .args[["max_iter"]], .args[["seed"]]
+    features_col = .args[["features_col"]],
+    k = .args[["k"]], max_iter = .args[["max_iter"]], seed = .args[["seed"]]
   ) %>%
     invoke("setPredictionCol", .args[["prediction_col"]]) %>%
     invoke("setMinDivisibleClusterSize", .args[["min_divisible_cluster_size"]])
@@ -83,7 +85,7 @@ ml_bisecting_kmeans.tbl_spark <- function(x, formula = NULL, k = 4, max_iter = 2
                                           features_col = "features", prediction_col = "prediction",
                                           uid = random_string("bisecting_bisecting_kmeans_"),
                                           features = NULL, ...) {
-  ml_formula_transformation()
+  formula <- ml_standardize_formula(formula, features = features)
 
   stage <- ml_bisecting_kmeans.spark_connection(
     x = spark_connection(x),
@@ -102,12 +104,17 @@ ml_bisecting_kmeans.tbl_spark <- function(x, formula = NULL, k = 4, max_iter = 2
     stage %>%
       ml_fit(x)
   } else {
-    ml_generate_ml_model(x, predictor = stage, formula = formula, features_col = features_col,
-                         type = "clustering", constructor = new_ml_model_bisecting_kmeans)
+    ml_construct_model_clustering(
+      new_ml_model_bisecting_kmeans,
+      predictor = stage,
+      dataset = x,
+      formula = formula,
+      features_col = features_col
+    )
   }
 }
 
-ml_validator_bisecting_kmeans <- function(.args) {
+validator_ml_bisecting_kmeans <- function(.args) {
   .args <- validate_args_clustering(.args)
   .args[["prediction_col"]] <- cast_string(.args[["prediction_col"]])
   .args[["min_divisible_cluster_size"]] <- cast_scalar_double(.args[["min_divisible_cluster_size"]])
@@ -115,7 +122,7 @@ ml_validator_bisecting_kmeans <- function(.args) {
 }
 
 new_ml_bisecting_kmeans <- function(jobj) {
-  new_ml_predictor(jobj, subclass = "ml_bisecting_kmeans")
+  new_ml_estimator(jobj, class = "ml_bisecting_kmeans")
 }
 
 new_ml_bisecting_kmeans_model <- function(jobj) {
@@ -126,17 +133,19 @@ new_ml_bisecting_kmeans_model <- function(jobj) {
 
   new_ml_clustering_model(
     jobj,
-    cluster_centers = function() try_null(invoke(jobj, "clusterCenters")) %>%
-      lapply(invoke, "toArray"),
+    cluster_centers = possibly_null(
+      ~ invoke(jobj, "clusterCenters") %>%
+        lapply(invoke, "toArray")
+    ),
     compute_cost = function(dataset) {
       invoke(jobj, "computeCost", spark_dataframe(dataset))
     },
     summary = summary,
-    subclass = "ml_bisecting_kmeans_model")
+    class = "ml_bisecting_kmeans_model")
 }
 
 new_ml_summary_bisecting_kmeans_model <- function(jobj) {
   new_ml_summary_clustering(
     jobj,
-    subclass = "ml_summary_bisecting_kmeans")
+    class = "ml_summary_bisecting_kmeans")
 }
