@@ -70,6 +70,7 @@
 #'
 #'
 #' @template roxlate-ml-clustering-algo
+#' @template roxlate-ml-formula-params
 #' @template roxlate-ml-clustering-params
 #' @param doc_concentration Concentration parameter (commonly named "alpha") for the prior placed on documents' distributions over topics ("theta"). See details.
 #' @param topic_concentration Concentration parameter (commonly named "beta" or "eta") for the prior placed on topics' distributions over terms.
@@ -80,20 +81,48 @@
 #' @param learning_decay (For Online optimizer only) Learning rate, set as an exponential decay rate. This should be between (0.5, 1.0] to guarantee asymptotic convergence. This is called "kappa" in the Online LDA paper (Hoffman et al., 2010). Default: 0.51, based on Hoffman et al.
 #' @param learning_offset (For Online optimizer only) A (positive) learning parameter that downweights early iterations. Larger values make early iterations count less. This is called "tau0" in the Online LDA paper (Hoffman et al., 2010) Default: 1024, following Hoffman et al.
 #' @param optimize_doc_concentration (For Online optimizer only) Indicates whether the \code{doc_concentration} (Dirichlet parameter for document-topic distribution) will be optimized during training. Setting this to true will make the model more expressive and fit the training data better. Default: \code{FALSE}
-#'
 #' @param keep_last_checkpoint (Spark 2.0.0+) (For EM optimizer only) If using checkpointing, this indicates whether to keep the last checkpoint. If \code{FALSE}, then the checkpoint will be deleted. Deleting the checkpoint can cause failures if a data partition is lost, so set this bit with care. Note that checkpoints will be cleaned up via reference counting, regardless.
 #'
+#' @examples
+#' \dontrun{
+#' library(janeaustenr)
+#' library(dplyr)
+#' sc <-  spark_connect(master = "local")
+#'
+#' lines_tbl <- sdf_copy_to(sc,
+#'                          austen_books()[c(1:30),],
+#'                          name = "lines_tbl",
+#'                          overwrite = TRUE)
+#'
+#' # transform the data in a tidy form
+#' lines_tbl_tidy <- lines_tbl %>%
+#'   ft_tokenizer(input_col = "text",
+#'                output_col = "word_list") %>%
+#'   ft_stop_words_remover(input_col = "word_list",
+#'                         output_col = "wo_stop_words") %>%
+#'   mutate(text = explode(wo_stop_words)) %>%
+#'   filter(text != "") %>%
+#'   select(text, book)
+#'
+#' lda_model <- lines_tbl_tidy %>%
+#'   ml_lda(~text, k = 4)
+#'
+#' # vocabulary and topics
+#' tidy(lda_model)
+#' }
+#'
 #' @export
-ml_lda <- function(x, k = 10, max_iter = 20, doc_concentration = NULL, topic_concentration = NULL,
+ml_lda <- function(x, formula = NULL, k = 10, max_iter = 20, doc_concentration = NULL, topic_concentration = NULL,
                    subsampling_rate = 0.05, optimizer = "online", checkpoint_interval = 10,
                    keep_last_checkpoint = TRUE, learning_decay = 0.51, learning_offset = 1024,
                    optimize_doc_concentration = TRUE, seed = NULL, features_col = "features",
                    topic_distribution_col = "topicDistribution", uid = random_string("lda_"), ...) {
+  check_dots_used()
   UseMethod("ml_lda")
 }
 
 #' @export
-ml_lda.spark_connection <- function(x, k = 10, max_iter = 20, doc_concentration = NULL, topic_concentration = NULL,
+ml_lda.spark_connection <- function(x, formula = NULL, k = 10, max_iter = 20, doc_concentration = NULL, topic_concentration = NULL,
                                     subsampling_rate = 0.05, optimizer = "online", checkpoint_interval = 10,
                                     keep_last_checkpoint = TRUE, learning_decay = 0.51, learning_offset = 1024,
                                     optimize_doc_concentration = TRUE, seed = NULL, features_col = "features",
@@ -116,7 +145,7 @@ ml_lda.spark_connection <- function(x, k = 10, max_iter = 20, doc_concentration
     topic_distribution_col = topic_distribution_col
   ) %>%
     c(rlang::dots_list(...)) %>%
-    ml_validator_lda()
+    validator_ml_lda()
 
   uid <- cast_string(uid)
 
@@ -126,21 +155,21 @@ ml_lda.spark_connection <- function(x, k = 10, max_iter = 20, doc_concentration
     invoke("setSubsamplingRate", .args[["subsampling_rate"]]) %>%
     invoke("setOptimizer", .args[["optimizer"]]) %>%
     invoke("setCheckpointInterval", .args[["checkpoint_interval"]]) %>%
-    maybe_set_param("setKeepLastCheckpoint", .args[["keep_last_checkpoint"]], "2.0.0", TRUE) %>%
+    jobj_set_param("setKeepLastCheckpoint", .args[["keep_last_checkpoint"]], "2.0.0", TRUE) %>%
     invoke("setLearningDecay", .args[["learning_decay"]]) %>%
     invoke("setLearningOffset", .args[["learning_offset"]]) %>%
     invoke("setOptimizeDocConcentration", .args[["optimize_doc_concentration"]]) %>%
     invoke("setFeaturesCol", .args[["features_col"]]) %>%
     invoke("setTopicDistributionCol", .args[["topic_distribution_col"]]) %>%
-    maybe_set_param("setDocConcentration", .args[["doc_concentration"]]) %>%
-    maybe_set_param("setTopicConcentration", .args[["topic_concentration"]]) %>%
-    maybe_set_param("setSeed", .args[["seed"]])
+    jobj_set_param("setDocConcentration", .args[["doc_concentration"]]) %>%
+    jobj_set_param("setTopicConcentration", .args[["topic_concentration"]]) %>%
+    jobj_set_param("setSeed", .args[["seed"]])
 
   new_ml_lda(jobj)
 }
 
 #' @export
-ml_lda.ml_pipeline <- function(x, k = 10, max_iter = 20, doc_concentration = NULL, topic_concentration = NULL,
+ml_lda.ml_pipeline <- function(x, formula = NULL, k = 10, max_iter = 20, doc_concentration = NULL, topic_concentration = NULL,
                                subsampling_rate = 0.05, optimizer = "online", checkpoint_interval = 10,
                                keep_last_checkpoint = TRUE, learning_decay = 0.51, learning_offset = 1024,
                                optimize_doc_concentration = TRUE, seed = NULL, features_col = "features",
@@ -148,6 +177,7 @@ ml_lda.ml_pipeline <- function(x, k = 10, max_iter = 20, doc_concentration = NUL
 
   stage <- ml_lda.spark_connection(
     x = spark_connection(x),
+    formula = formula,
     k = k,
     max_iter = max_iter,
     doc_concentration = doc_concentration,
@@ -169,13 +199,16 @@ ml_lda.ml_pipeline <- function(x, k = 10, max_iter = 20, doc_concentration = NUL
 }
 
 #' @export
-ml_lda.tbl_spark <- function(x, k = 10, max_iter = 20, doc_concentration = NULL, topic_concentration = NULL,
+ml_lda.tbl_spark <- function(x, formula = NULL, k = 10, max_iter = 20, doc_concentration = NULL, topic_concentration = NULL,
                              subsampling_rate = 0.05, optimizer = "online", checkpoint_interval = 10,
                              keep_last_checkpoint = TRUE, learning_decay = 0.51, learning_offset = 1024,
                              optimize_doc_concentration = TRUE, seed = NULL, features_col = "features",
                              topic_distribution_col = "topicDistribution", uid = random_string("lda_"), ...) {
-  stage <- ml_lda.spark_connection(
+  formula <- ml_standardize_formula(formula)
+
+   stage <- ml_lda.spark_connection(
     x = spark_connection(x),
+    formula = NULL,
     k = k,
     max_iter = max_iter,
     doc_concentration = doc_concentration,
@@ -194,18 +227,23 @@ ml_lda.tbl_spark <- function(x, k = 10, max_iter = 20, doc_concentration = NULL,
     ...
   )
 
+  if (is.null(formula)) {
   stage %>%
     ml_fit(x)
+   } else {
+     ml_construct_model_clustering(
+       new_ml_model_lda,
+       predictor = stage,
+       dataset = x,
+       formula = formula,
+       features_col = features_col
+     )
+   }
 }
 
 # Validator
-ml_validator_lda <- function(.args) {
-  .args <- ml_backwards_compatibility(.args, list(
-    alpha = "doc_concentration",
-    beta = "topic_concentration",
-    max.iterations = "max_iter"
-  )) %>%
-    validate_args_clustering()
+validator_ml_lda <- function(.args) {
+  .args <- validate_args_clustering(.args)
 
   .args[["doc_concentration"]] <- cast_nullable_scalar_double(.args[["doc_concentration"]])
   .args[["topic_concentration"]] <- cast_nullable_scalar_double(.args[["topic_concentration"]])
@@ -221,7 +259,7 @@ ml_validator_lda <- function(.args) {
 }
 
 new_ml_lda <- function(jobj) {
-  new_ml_predictor(jobj, subclass = "ml_lda")
+  new_ml_estimator(jobj, class = "ml_lda")
 }
 
 new_ml_lda_model <- function(jobj) {
@@ -233,17 +271,17 @@ new_ml_lda_model <- function(jobj) {
       invoke(jobj, "describeTopics", max_terms_per_topic) %>%
         sdf_register()
     },
-    estimated_doc_concentration = function() try_null(invoke(jobj, "estimatedDocConcentration")), # def
+    estimated_doc_concentration = possibly_null(~ invoke(jobj, "estimatedDocConcentration")), # def
     log_likelihood = function(dataset) invoke(jobj, "logLikelihood", spark_dataframe(dataset)),
     log_perplexity = function(dataset) invoke(jobj, "logPerplexity", spark_dataframe(dataset)),
     # topicsMatrix deprecated
     topicsMatrix = function() {
       warning("`topicMatrix()` is deprecated; please use `topics_matrix()` instead.")
-      try_null(read_spark_matrix(jobj, "topicsMatrix")) # def
+      possibly_null(read_spark_matrix)(jobj, "topicsMatrix") # def
     },
-    topics_matrix = function() try_null(read_spark_matrix(jobj, "topicsMatrix")), # def
+    topics_matrix = possibly_null(~ read_spark_matrix(jobj, "topicsMatrix")), # def
     vocab_size = invoke(jobj, "vocabSize"),
-    subclass = "ml_lda_model")
+    class = "ml_lda_model")
 }
 
 #' @rdname ml_lda
