@@ -33,9 +33,9 @@
 #' gmm_model <- ml_gaussian_mixture(iris_training, formula = formula)
 #'
 #' # Predict
-#' pred_kmeans <- sdf_predict(iris_test, kmeans_model)
-#' pred_b_kmeans <- sdf_predict(iris_test, b_kmeans_model)
-#' pred_gmm <- sdf_predict(iris_test, gmm_model)
+#' pred_kmeans <- ml_predict(kmeans_model, iris_test)
+#' pred_b_kmeans <- ml_predict(b_kmeans_model, iris_test)
+#' pred_gmm <- ml_predict(gmm_model, iris_test)
 #'
 #' # Evaluate
 #' ml_clustering_evaluator(pred_kmeans)
@@ -58,9 +58,9 @@ ml_clustering_evaluator.spark_connection <- function(x, features_col = "features
     prediction_col = prediction_col,
     metric_name = metric_name
   ) %>%
-    ml_validator_clustering_evaluator()
+    validator_ml_clustering_evaluator()
 
-  evaluator <- ml_new_identifiable(x, "org.apache.spark.ml.evaluation.ClusteringEvaluator", uid) %>%
+  evaluator <- spark_pipeline_stage(x, "org.apache.spark.ml.evaluation.ClusteringEvaluator", uid) %>%
     invoke("setFeaturesCol", .args[["features_col"]]) %>%
     invoke("setPredictionCol", .args[["prediction_col"]]) %>%
     invoke("setMetricName", .args[["metric_name"]]) %>%
@@ -86,7 +86,7 @@ ml_clustering_evaluator.tbl_spark <- function(x, features_col = "features", pred
 }
 
 # Validator
-ml_validator_clustering_evaluator <- function(.args) {
+validator_ml_clustering_evaluator <- function(.args) {
   .args[["features_col"]] <- cast_string(.args[["features_col"]])
   .args[["prediction_col"]] <- cast_string(.args[["prediction_col"]])
   .args[["metric_name"]] <- cast_choice(.args[["metric_name"]], "silhouette")
