@@ -17,14 +17,17 @@
 #'   value of the feature. Default: mean
 #' @export
 ft_imputer <- function(x, input_cols = NULL, output_cols = NULL,
-                       missing_value = NULL, strategy = "mean", dataset = NULL,
+                       missing_value = NULL, strategy = "mean",
                        uid = random_string("imputer_"), ...) {
+  check_dots_used()
   UseMethod("ft_imputer")
 }
 
+ml_imputer <- ft_imputer
+
 #' @export
 ft_imputer.spark_connection <- function(x, input_cols = NULL, output_cols = NULL,
-                                        missing_value = NULL, strategy = "mean", dataset = NULL,
+                                        missing_value = NULL, strategy = "mean",
                                         uid = random_string("imputer_"), ...) {
   spark_require_version(x, "2.2.0", "Imputer")
 
@@ -36,25 +39,22 @@ ft_imputer.spark_connection <- function(x, input_cols = NULL, output_cols = NULL
     uid = uid
   ) %>%
     c(rlang::dots_list(...)) %>%
-    ml_validator_imputer()
+    validator_ml_imputer()
 
-  jobj <- ml_new_transformer(
+  jobj <- spark_pipeline_stage(
     x, "org.apache.spark.ml.feature.Imputer",
     input_cols = .args[["input_cols"]], output_cols = .args[["output_cols"]], uid = .args[["uid"]]) %>%
     invoke("setStrategy", .args[["strategy"]]) %>%
-    maybe_set_param("setMissingValue", .args[["missing_value"]])
+    jobj_set_param("setMissingValue", .args[["missing_value"]])
 
   estimator <- new_ml_imputer(jobj)
 
-  if (is.null(dataset))
-    estimator
-  else
-    ml_fit(estimator, dataset)
+  estimator
 }
 
 #' @export
 ft_imputer.ml_pipeline <- function(x, input_cols = NULL, output_cols = NULL,
-                                   missing_value = NULL, strategy = "mean", dataset = NULL,
+                                   missing_value = NULL, strategy = "mean",
                                    uid = random_string("imputer_"), ...) {
   stage <- ft_imputer.spark_connection(
     x = spark_connection(x),
@@ -62,7 +62,6 @@ ft_imputer.ml_pipeline <- function(x, input_cols = NULL, output_cols = NULL,
     output_cols = output_cols,
     missing_value = missing_value,
     strategy = strategy,
-    dataset = dataset,
     uid = uid,
     ...
   )
@@ -71,7 +70,7 @@ ft_imputer.ml_pipeline <- function(x, input_cols = NULL, output_cols = NULL,
 
 #' @export
 ft_imputer.tbl_spark <- function(x, input_cols = NULL, output_cols = NULL,
-                                 missing_value = NULL, strategy = "mean", dataset = NULL,
+                                 missing_value = NULL, strategy = "mean",
                                  uid = random_string("imputer_"), ...) {
   stage <- ft_imputer.spark_connection(
     x = spark_connection(x),
@@ -79,7 +78,6 @@ ft_imputer.tbl_spark <- function(x, input_cols = NULL, output_cols = NULL,
     output_cols = output_cols,
     missing_value = missing_value,
     strategy = strategy,
-    dataset = dataset,
     uid = uid,
     ...
   )
@@ -89,7 +87,7 @@ ft_imputer.tbl_spark <- function(x, input_cols = NULL, output_cols = NULL,
     ml_fit_and_transform(stage, x)
 }
 
-ml_validator_imputer <- function(.args) {
+validator_ml_imputer <- function(.args) {
   .args[["input_cols"]] <- cast_nullable_string_list(.args[["input_cols"]])
   .args[["output_cols"]] <- cast_nullable_string_list(.args[["output_cols"]])
   .args[["strategy"]] <- cast_choice(.args[["strategy"]], c("mean", "median"))
@@ -98,9 +96,9 @@ ml_validator_imputer <- function(.args) {
 }
 
 new_ml_imputer <- function(jobj) {
-  new_ml_estimator(jobj, subclass = "ml_imputer")
+  new_ml_estimator(jobj, class = "ml_imputer")
 }
 
 new_ml_imputer_model <- function(jobj) {
-  new_ml_transformer(jobj, subclass = "ml_imputer_model")
+  new_ml_transformer(jobj, class = "ml_imputer_model")
 }
