@@ -23,13 +23,16 @@
 #' @template roxlate-ml-feature-estimator-transformer
 #' @export
 ft_max_abs_scaler <- function(x, input_col = NULL, output_col = NULL,
-                              dataset = NULL,uid = random_string("max_abs_scaler_"), ...) {
+                              uid = random_string("max_abs_scaler_"), ...) {
+  check_dots_used()
   UseMethod("ft_max_abs_scaler")
 }
 
+ml_max_abs_scaler <- ft_max_abs_scaler
+
 #' @export
 ft_max_abs_scaler.spark_connection <- function(x, input_col = NULL, output_col = NULL,
-                                               dataset = NULL,uid = random_string("max_abs_scaler_"), ...) {
+                                               uid = random_string("max_abs_scaler_"), ...) {
   spark_require_version(x, "2.0.0", "MaxAbsScaler")
 
   .args <- list(
@@ -38,29 +41,26 @@ ft_max_abs_scaler.spark_connection <- function(x, input_col = NULL, output_col =
     uid = uid
   ) %>%
     c(rlang::dots_list(...)) %>%
-    ml_validator_max_abs_scaler()
+    validator_ml_max_abs_scaler()
 
-  estimator <- ml_new_transformer(
+  estimator <- spark_pipeline_stage(
     x, "org.apache.spark.ml.feature.MaxAbsScaler",
     input_col = .args[["input_col"]], output_col = .args[["output_col"]], uid = .args[["uid"]]
   ) %>%
     new_ml_max_abs_scaler()
 
-  if (is.null(dataset))
-    estimator
-  else
-    ml_fit(estimator, dataset)
+
+  estimator
 }
 
 #' @export
 ft_max_abs_scaler.ml_pipeline <- function(x, input_col = NULL, output_col = NULL,
-                                          dataset = NULL,uid = random_string("max_abs_scaler_"), ...) {
+                                          uid = random_string("max_abs_scaler_"), ...) {
 
   stage <- ft_max_abs_scaler.spark_connection(
     x = spark_connection(x),
     input_col = input_col,
     output_col = output_col,
-    dataset = dataset,
     uid = uid,
     ...
   )
@@ -70,13 +70,12 @@ ft_max_abs_scaler.ml_pipeline <- function(x, input_col = NULL, output_col = NULL
 
 #' @export
 ft_max_abs_scaler.tbl_spark <- function(x, input_col = NULL, output_col = NULL,
-                                        dataset = NULL,uid = random_string("max_abs_scaler_"), ...) {
+                                        uid = random_string("max_abs_scaler_"), ...) {
 
   stage <- ft_max_abs_scaler.spark_connection(
     x = spark_connection(x),
     input_col = input_col,
     output_col = output_col,
-    dataset = dataset,
     uid = uid,
     ...
   )
@@ -88,13 +87,13 @@ ft_max_abs_scaler.tbl_spark <- function(x, input_col = NULL, output_col = NULL,
 }
 
 new_ml_max_abs_scaler <- function(jobj) {
-  new_ml_estimator(jobj, subclass = "ml_max_abs_scaler")
+  new_ml_estimator(jobj, class = "ml_max_abs_scaler")
 }
 
 new_ml_max_abs_scaler_model <- function(jobj) {
-  new_ml_transformer(jobj, subclass = "ml_max_abs_scaler_model")
+  new_ml_transformer(jobj, class = "ml_max_abs_scaler_model")
 }
 
-ml_validator_max_abs_scaler <- function(.args) {
+validator_ml_max_abs_scaler <- function(.args) {
   validate_args_transformer(.args)
 }
