@@ -29,15 +29,19 @@
 #'
 #' @export
 ft_standard_scaler <- function(x, input_col = NULL, output_col = NULL,
-                               with_mean = FALSE, with_std = TRUE, dataset = NULL,
+                               with_mean = FALSE, with_std = TRUE,
                                uid = random_string("standard_scaler_"), ...) {
+  check_dots_used()
   UseMethod("ft_standard_scaler")
 }
 
+ml_standard_scaler <- ft_standard_scaler
+
 #' @export
 ft_standard_scaler.spark_connection <- function(x, input_col = NULL, output_col = NULL,
-                                                with_mean = FALSE, with_std = TRUE, dataset = NULL,
+                                                with_mean = FALSE, with_std = TRUE,
                                                 uid = random_string("standard_scaler_"), ...) {
+
   .args <- list(
     input_col = input_col,
     output_col = output_col,
@@ -46,9 +50,9 @@ ft_standard_scaler.spark_connection <- function(x, input_col = NULL, output_col
     uid = uid
   ) %>%
     c(rlang::dots_list(...)) %>%
-    ml_validator_standard_scaler()
+    validator_ml_standard_scaler()
 
-  estimator <- ml_new_transformer(
+  estimator <- spark_pipeline_stage(
     x, "org.apache.spark.ml.feature.StandardScaler",
     input_col = .args[["input_col"]], output_col = .args[["output_col"]], uid = .args[["uid"]]
   ) %>%
@@ -56,15 +60,12 @@ ft_standard_scaler.spark_connection <- function(x, input_col = NULL, output_col
     invoke("setWithStd", .args[["with_std"]]) %>%
     new_ml_standard_scaler()
 
-  if (is.null(dataset))
-    estimator
-  else
-    ml_fit(estimator, dataset)
+  estimator
 }
 
 #' @export
 ft_standard_scaler.ml_pipeline <- function(x, input_col = NULL, output_col = NULL,
-                                           with_mean = FALSE, with_std = TRUE, dataset = NULL,
+                                           with_mean = FALSE, with_std = TRUE,
                                            uid = random_string("standard_scaler_"), ...) {
   stage <- ft_standard_scaler.spark_connection(
     x = spark_connection(x),
@@ -72,7 +73,6 @@ ft_standard_scaler.ml_pipeline <- function(x, input_col = NULL, output_col = NUL
     output_col = output_col,
     with_mean = with_mean,
     with_std = with_std,
-    dataset = dataset,
     uid = uid,
     ...
   )
@@ -82,7 +82,7 @@ ft_standard_scaler.ml_pipeline <- function(x, input_col = NULL, output_col = NUL
 
 #' @export
 ft_standard_scaler.tbl_spark <- function(x, input_col = NULL, output_col = NULL,
-                                         with_mean = FALSE, with_std = TRUE, dataset = NULL,
+                                         with_mean = FALSE, with_std = TRUE,
                                          uid = random_string("standard_scaler_"), ...) {
   stage <- ft_standard_scaler.spark_connection(
     x = spark_connection(x),
@@ -90,7 +90,6 @@ ft_standard_scaler.tbl_spark <- function(x, input_col = NULL, output_col = NULL,
     output_col = output_col,
     with_mean = with_mean,
     with_std = with_std,
-    dataset = dataset,
     uid = uid,
     ...
   )
@@ -102,18 +101,18 @@ ft_standard_scaler.tbl_spark <- function(x, input_col = NULL, output_col = NULL,
 }
 
 new_ml_standard_scaler <- function(jobj) {
-  new_ml_estimator(jobj, subclass = "ml_standard_scaler")
+  new_ml_estimator(jobj, class = "ml_standard_scaler")
 }
 
 new_ml_standard_scaler_model <- function(jobj) {
   new_ml_transformer(
     jobj,
-    mean = try_null(read_spark_vector(jobj, "mean")),
-    std = try_null(read_spark_vector(jobj, "std")),
-    subclass = "ml_standard_scaler_model")
+    mean = possibly_null(read_spark_vector)(jobj, "mean"),
+    std = possibly_null(read_spark_vector)(jobj, "std"),
+    class = "ml_standard_scaler_model")
 }
 
-ml_validator_standard_scaler <- function(.args) {
+validator_ml_standard_scaler <- function(.args) {
   .args <- validate_args_transformer(.args)
   .args[["with_mean"]] <- cast_scalar_logical(.args[["with_mean"]])
   .args[["with_std"]] <- cast_scalar_logical(.args[["with_std"]])
