@@ -34,7 +34,7 @@ NULL
 #' rf_model <- iris_training %>%
 #'   ml_random_forest(Species ~ ., type = "classification")
 #'
-#' pred <- sdf_predict(iris_test, rf_model)
+#' pred <- ml_predict(rf_model, iris_test)
 #'
 #' ml_multiclass_classification_evaluator(pred)
 #' }
@@ -47,94 +47,110 @@ ml_random_forest <- function(x, formula = NULL, type = c("auto", "regression", "
                              min_instances_per_node = 1, subsampling_rate = 1, seed = NULL,
                              thresholds = NULL, cache_node_ids = FALSE, max_memory_in_mb = 256,
                              uid = random_string("random_forest_"), response = NULL, features = NULL, ...) {
-  ml_formula_transformation()
+  formula <- ml_standardize_formula(formula, response, features)
   response_col <- gsub("~.+$", "", formula) %>% trimws()
 
   sdf <- spark_dataframe(x)
   # choose classification vs. regression model based on column type
   schema <- sdf_schema(sdf)
-  if (!response_col %in% names(schema))
-    stop(paste0("`", response_col, "` is not a column in the input dataset."))
+  if (!response_col %in% names(schema)) {
+    stop("`", response_col, "` is not a column in the input dataset.")
+  }
 
   response_type <- schema[[response_col]]$type
 
   type <- rlang::arg_match(type)
-  model_type <- if (!identical(type, "auto")) type else {
-    if (response_type %in% c("DoubleType", "IntegerType"))
+  model_type <- if (!identical(type, "auto")) {
+    type
+  } else {
+    if (response_type %in% c("DoubleType", "IntegerType")) {
       "regression"
-    else
+    } else {
       "classification"
+    }
   }
 
   impurity <- if (identical(impurity, "auto")) {
     if (identical(model_type, "regression")) "variance" else "gini"
   } else if (identical(model_type, "classification")) {
-    if (!impurity %in% c("gini", "entropy"))
+    if (!impurity %in% c("gini", "entropy")) {
       stop("`impurity` must be \"gini\" or \"entropy\" for classification.")
+    }
     impurity
   } else {
-    if (!identical(impurity, "variance"))
+    if (!identical(impurity, "variance")) {
       stop("`impurity` must be \"variance\" for regression.")
+    }
     impurity
   }
 
-  if (rlang::has_name(rlang::dots_list(...), "col.sample.rate")) {
-    col.sample.rate <- rlang::dots_list(...)[["col.sample.rate"]]
-    sc <- spark_connection(x)
-    if (!(col.sample.rate > 0 && col.sample.rate <= 1))
-      stop("`col.sample.rate` must be in (0, 1].")
-    col.sample.rate <- if (spark_version(sc) < "2.0.0") {
-      if (col.sample.rate == 1) # nocov start
-        "all"
-      else {
-        # Prior to Spark 2.0.0, random forest does not support arbitrary
-        #   column sampling rates. So we map the input to one of the supported
-        #   strategies: "onethird", "sqrt", or "log2".
-        k <- gsub("^.+~", "", formula) %>%
-          trimws() %>%
-          strsplit("\\+") %>%
-          rlang::flatten_chr() %>%
-          length()
-        strategies <- dplyr::data_frame(strategy = c("onethird", "sqrt", "log2"),
-                                        rate = c(1/3, sqrt(k)/k, log2(k)/k)) %>%
-          dplyr::arrange(!! rlang::sym("rate"))
-        strategy <- strategies %>%
-          dplyr::pull("strategy") %>%
-          `[[`(max(findInterval(col.sample.rate, strategies[["rate"]]), 1))
-        message("* Using feature subsetting strategy: ", strategy)
-        strategy
-      } # nocov end
-    } else {
-      cast_string(format(col.sample.rate, nsmall = 1))
-    }
-    assign("col.sample.rate", col.sample.rate, envir = parent.frame())
-  }
-
-  routine <- switch(model_type,
-                    regression = ml_random_forest_regressor,
-                    classification = ml_random_forest_classifier)
-
-  args <- c(as.list(environment()), list(...))
-  args$response <- NULL
-  args$features <- NULL
-  do.call(routine, args)
+  switch(
+    model_type,
+    regression = ml_random_forest_regressor(
+      x = x,
+      formula = formula,
+      num_trees = num_trees,
+      subsampling_rate = subsampling_rate,
+      max_depth = max_depth,
+      min_instances_per_node = min_instances_per_node,
+      feature_subset_strategy = feature_subset_strategy,
+      impurity = impurity,
+      min_info_gain = min_info_gain,
+      max_bins = max_bins,
+      seed = seed,
+      checkpoint_interval = checkpoint_interval,
+      cache_node_ids = cache_node_ids,
+      max_memory_in_mb = max_memory_in_mb,
+      features_col = features_col,
+      label_col = label_col,
+      prediction_col = prediction_col,
+      uid = uid,
+      ...
+    ),
+    classification = ml_random_forest_classifier(
+      x = x,
+      formula = formula,
+      num_trees = num_trees,
+      subsampling_rate = subsampling_rate,
+      max_depth = max_depth,
+      min_instances_per_node = min_instances_per_node,
+      feature_subset_strategy = feature_subset_strategy,
+      impurity = impurity,
+      min_info_gain = min_info_gain,
+      max_bins = max_bins,
+      seed = seed,
+      thresholds = thresholds,
+      checkpoint_interval = checkpoint_interval,
+      cache_node_ids = cache_node_ids,
+      max_memory_in_mb = max_memory_in_mb,
+      features_col = features_col,
+      label_col = label_col,
+      prediction_col = prediction_col,
+      probability_col = probability_col,
+      raw_prediction_col = raw_prediction_col,
+      uid = uid,
+      ...
+    )
+  )
 }
 
-new_ml_model_random_forest_classification <- function(pipeline, pipeline_model, model, dataset,
-                                                      formula, feature_names, index_labels, call) {
+new_ml_model_random_forest_classification <- function(pipeline_model, formula, dataset, label_col,
+                                                      features_col, predicted_label_col) {
   new_ml_model_classification(
-    pipeline, pipeline_model, model, dataset, formula,
-    subclass = "ml_model_random_forest_classification",
-    .features = feature_names,
-    .index_labels = index_labels
+    pipeline_model, formula,
+    dataset = dataset,
+    label_col = label_col, features_col = features_col,
+    predicted_label_col = predicted_label_col,
+    class = "ml_model_random_forest_classification"
   )
 }
 
-new_ml_model_random_forest_regression <- function(pipeline, pipeline_model, model, dataset, formula,
-                                                  feature_names, call) {
+new_ml_model_random_forest_regression <- function(pipeline_model, formula, dataset, label_col,
+                                                  features_col) {
   new_ml_model_regression(
-    pipeline, pipeline_model, model, dataset, formula,
-    subclass = "ml_model_random_forest_regression",
-    .features = feature_names
+    pipeline_model, formula,
+    dataset = dataset,
+    label_col = label_col, features_col = features_col,
+    class = "ml_model_random_forest_regression"
   )
 }
