@@ -23,7 +23,7 @@
 #' iso_res <- iris_tbl %>%
 #'   ml_isotonic_regression(Petal_Length ~ Petal_Width)
 #'
-#' pred <- sdf_predict(iris_test, iso_res)
+#' pred <- ml_predict(iso_res, iris_test)
 #'
 #' pred
 #' }
@@ -33,6 +33,7 @@ ml_isotonic_regression <- function(x, formula = NULL, feature_index = 0, isotoni
                                    weight_col = NULL, features_col = "features",
                                    label_col = "label", prediction_col = "prediction",
                                    uid = random_string("isotonic_regression_"), ...) {
+  check_dots_used()
   UseMethod("ml_isotonic_regression")
 }
 
@@ -51,15 +52,16 @@ ml_isotonic_regression.spark_connection <- function(x, formula = NULL, feature_i
     prediction_col = prediction_col
   ) %>%
     c(rlang::dots_list(...)) %>%
-    ml_validator_isotonic_regression()
+    validator_ml_isotonic_regression()
 
-  jobj <- ml_new_predictor(
+  jobj <- spark_pipeline_stage(
     x, "org.apache.spark.ml.regression.IsotonicRegression", uid,
-    .args[["features_col"]], .args[["label_col"]], .args[["prediction_col"]]
+    features_col = .args[["features_col"]], label_col = .args[["label_col"]],
+    prediction_col = .args[["prediction_col"]]
   ) %>%
     invoke("setFeatureIndex", .args[["feature_index"]]) %>%
     invoke("setIsotonic", .args[["isotonic"]]) %>%
-    maybe_set_param("setWeightCol", .args[["weight_col"]])
+    jobj_set_param("setWeightCol", .args[["weight_col"]])
 
   new_ml_isotonic_regression(jobj)
 }
@@ -91,7 +93,7 @@ ml_isotonic_regression.tbl_spark <- function(x, formula = NULL, feature_index =
                                              label_col = "label", prediction_col = "prediction",
                                              uid = random_string("isotonic_regression_"),
                                              response = NULL, features = NULL, ...) {
-  ml_formula_transformation()
+  formula <- ml_standardize_formula(formula, response, features)
 
   stage <- ml_isotonic_regression.spark_connection(
     x = spark_connection(x),
@@ -110,13 +112,19 @@ ml_isotonic_regression.tbl_spark <- function(x, formula = NULL, feature_index =
     stage %>%
       ml_fit(x)
   } else {
-    ml_generate_ml_model(x, stage, formula, features_col, label_col,
-                         "regression", new_ml_model_isotonic_regression)
+    ml_construct_model_supervised(
+      new_ml_model_isotonic_regression,
+      predictor = stage,
+      formula = formula,
+      dataset = x,
+      features_col = features_col,
+      label_col = label_col
+    )
   }
 }
 
 # Validator
-ml_validator_isotonic_regression <- function(.args) {
+validator_ml_isotonic_regression <- function(.args) {
   .args[["feature_index"]] <- cast_scalar_integer(.args[["feature_index"]])
   .args[["isotonic"]] <- cast_scalar_logical(.args[["isotonic"]])
   .args[["weight_col"]] <- cast_nullable_string(.args[["weight_col"]])
@@ -124,16 +132,14 @@ ml_validator_isotonic_regression <- function(.args) {
 }
 
 new_ml_isotonic_regression <- function(jobj) {
-  new_ml_predictor(jobj, subclass = "ml_isotonic_regression")
+  new_ml_estimator(jobj, class = "ml_isotonic_regression")
 }
 
 new_ml_isotonic_regression_model <- function(jobj) {
-  new_ml_prediction_model(
+  new_ml_transformer(
     jobj,
     boundaries = function() read_spark_vector(jobj, "boundaries"), # lazy val
     predictions = function() read_spark_vector(jobj, "predictions"), # lazy val
     feature_index = invoke(jobj, "getFeatureIndex"),
-    features_col = invoke(jobj, "getFeaturesCol"),
-    prediction_col = invoke(jobj, "getPredictionCol"),
-    subclass = "ml_isotonic_regression_model")
+    class = "ml_isotonic_regression_model")
 }
