@@ -1,15 +1,15 @@
-new_ml_summary <- function(jobj, ..., subclass = NULL) {
+new_ml_summary <- function(jobj, ..., class = character()) {
   structure(
     list(
       type = jobj_info(jobj)$class,
       ...,
       .jobj = jobj
     ),
-    class = c(subclass, "ml_summary")
+    class = c(class, "ml_summary")
   )
 }
 
-new_ml_summary_clustering <- function(jobj, ..., subclass = NULL) {
+new_ml_summary_clustering <- function(jobj, ..., class = character()) {
   new_ml_summary(
     jobj,
     cluster = function() invoke(jobj, "cluster") %>% sdf_register(), # lazy val
@@ -18,6 +18,6 @@ new_ml_summary_clustering <- function(jobj, ..., subclass = NULL) {
     k = invoke(jobj, "k"),
     prediction_col = invoke(jobj, "predictionCol"),
     predictions = invoke(jobj, "predictions") %>% sdf_register(),
-    subclass = c(subclass, "ml_summary_clustering")
+    class = c(class, "ml_summary_clustering")
   )
 }
