@@ -32,13 +32,14 @@ spark_apply_packages_is_bundle <- function(packages) {
   is.character(packages) && length(packages) == 1 && grepl("\\.tar$", packages)
 }
 
-spark_apply_worker_config <- function(sc, debug, profile, schema = FALSE) {
+spark_apply_worker_config <- function(sc, debug, profile, schema = FALSE, arrow = FALSE) {
   worker_config_serialize(
     c(
       list(
         debug = isTRUE(debug),
         profile = isTRUE(profile),
-        schema = isTRUE(schema)
+        schema = isTRUE(schema),
+        arrow =isTRUE(arrow)
       ),
       sc$config
     )
@@ -134,12 +135,13 @@ spark_apply_colum_types <- function(sdf) {
 spark_apply <- function(x,
                         f,
                         columns = NULL,
-                        memory = TRUE,
+                        memory = !is.null(name),
                         group_by = NULL,
                         packages = NULL,
                         context = NULL,
                         name = NULL,
                         ...) {
+  memory <- force(memory)
   args <- list(...)
   assert_that(is.function(f) || is.raw(f) || is.language(f))
   if (is.language(f)) f <- rlang::as_closure(f)
@@ -156,7 +158,29 @@ spark_apply <- function(x,
 
   rlang <- spark_config_value(sc$config, "sparklyr.apply.rlang", FALSE)
   packages_config <- spark_config_value(sc$config, "sparklyr.apply.packages", NULL)
-  proc_env <- connection_config(sc, "sparklyr.apply.env.")
+  proc_env <- c(connection_config(sc, "sparklyr.apply.env."), args$env)
+
+  time_zone <- ""
+  records_per_batch <- NULL
+  arrow <- if (!is.null(args$arrow)) args$arrow else arrow_enabled(sc, sdf)
+  if (arrow) {
+    time_zone <- spark_session(sc) %>% invoke("sessionState") %>% invoke("conf") %>% invoke("sessionLocalTimeZone")
+    records_per_batch <- as.integer(spark_session_config(sc)[["spark.sql.execution.arrow.maxRecordsPerBatch"]] %||% 10000)
+  }
+
+  # build reduced size query plan in case schema needs to be inferred
+  if (sdf_is_streaming(sdf)) {
+    sdf_limit <- sdf
+  }
+  else {
+    sdf_limit <- invoke(
+      sdf,
+      "limit",
+      cast_scalar_integer(
+        spark_config_value(sc$config, "sparklyr.apply.schema.infer", 10)
+      )
+    )
+  }
 
   # backward compatible support for names argument from 0.6
   if (!is.null(args$names)) {
@@ -184,6 +208,9 @@ spark_apply <- function(x,
   # disable package distribution for local connections
   if (spark_master_is_local(sc$master)) packages <- FALSE
 
+  # disable package distribution for livy connections and no package spec
+  if (identical(tolower(sc$method), "livy") && identical(packages, TRUE)) packages <- FALSE
+
   # inject column types to context
   context <- list(
     column_types = spark_apply_colum_types(x),
@@ -222,8 +249,13 @@ spark_apply <- function(x,
     if (identical(args$rdd, TRUE)) {
       rdd_base <- invoke_static(sc, "sparklyr.ApplyUtils", "groupBy", rdd_base, group_by_list)
     }
+    else if (arrow) {
+      sdf <- invoke_static(sc, "sparklyr.ApplyUtils", "groupByArrow", sdf, group_by_list, time_zone, records_per_batch)
+      sdf_limit <- invoke_static(sc, "sparklyr.ApplyUtils", "groupByArrow", sdf_limit, group_by_list, time_zone, records_per_batch)
+    }
     else {
       sdf <- invoke_static(sc, "sparklyr.ApplyUtils", "groupBy", sdf, group_by_list)
+      sdf_limit <- invoke_static(sc, "sparklyr.ApplyUtils", "groupBy", sdf_limit, group_by_list)
     }
   }
 
@@ -259,6 +291,7 @@ spark_apply <- function(x,
     connection_config(sc, "sparklyr.apply.options."),
     as.character
   )
+  if (!is.null(records_per_batch)) spark_apply_options[["maxRecordsPerBatch"]] <- as.character(records_per_batch)
 
   if (identical(args$rdd, TRUE)) {
     rdd <- invoke_static(
@@ -296,19 +329,6 @@ spark_apply <- function(x,
         )
       )
 
-      if (sdf_is_streaming(sdf)) {
-        sdf_limit <- sdf
-      }
-      else {
-        sdf_limit <- invoke(
-          sdf,
-          "limit",
-          cast_scalar_integer(
-            spark_config_value(sc$config, "sparklyr.apply.schema.infer", 10)
-          )
-        )
-      }
-
       columns_op <- invoke_static(
         sc,
         "sparklyr.WorkerHelper",
@@ -316,7 +336,7 @@ spark_apply <- function(x,
         sdf_limit,
         columns_schema,
         closure,
-        spark_apply_worker_config(sc, FALSE, FALSE, schema = TRUE),
+        spark_apply_worker_config(sc, args$debug, args$profile, schema = TRUE, arrow = arrow),
         as.integer(worker_port),
         as.list(sdf_columns),
         as.list(group_by),
@@ -325,10 +345,15 @@ spark_apply <- function(x,
         as.environment(proc_env),
         as.integer(60),
         context_serialize,
-        as.environment(spark_apply_options)
+        as.environment(spark_apply_options),
+        spark_session(sc),
+        time_zone
       )
 
       columns_query <- columns_op %>% sdf_collect()
+      if (arrow && !arrow_enabled_dataframe_schema(columns_query$types)) {
+        arrow <- FALSE
+      }
 
       columns_infer <- strsplit(columns_query[1, ]$types, split = "\\|")[[1]]
       names(columns_infer) <- strsplit(columns_query[1, ]$names, split = "\\|")[[1]]
@@ -351,7 +376,7 @@ spark_apply <- function(x,
       sdf,
       schema,
       closure,
-      spark_apply_worker_config(sc, args$debug, args$profile),
+      spark_apply_worker_config(sc, args$debug, args$profile, arrow = arrow),
       as.integer(worker_port),
       as.list(sdf_columns),
       as.list(group_by),
@@ -360,7 +385,9 @@ spark_apply <- function(x,
       as.environment(proc_env),
       as.integer(60),
       context_serialize,
-      as.environment(spark_apply_options)
+      as.environment(spark_apply_options),
+      spark_session(sc),
+      time_zone
     )
   }
 
