@@ -24,16 +24,12 @@ ensure_round_trip <- function(sc, data) {
 }
 
 test_that("objects survive Spark roundtrips", {
-  skip_on_cran()
-
   datasets <- list(mtcars = mtcars)
   for (dataset in datasets)
     ensure_round_trip(sc, dataset)
 })
 
 test_that("primitive values survive Spark roundtrips", {
-  skip_on_cran()
-
   n <- 10
   df <- data.frame(
     int = as.integer(1:n),
@@ -48,8 +44,6 @@ test_that("primitive values survive Spark roundtrips", {
 })
 
 test_that("NA values survive Spark roundtrips", {
-  skip_on_cran()
-
   n <- 10
   df <- data.frame(
     int = as.integer(1:n),
@@ -65,8 +59,6 @@ test_that("NA values survive Spark roundtrips", {
 })
 
 test_that("data.frames with '|' can be copied", {
-  skip_on_cran()
-
   pipes <- data.frame(
     x = c("|||", "|||", "|||"),
     y = c(1, 2, 3),
@@ -77,7 +69,7 @@ test_that("data.frames with '|' can be copied", {
 })
 
 test_that("data.frames with many columns survive roundtrip", {
-  skip_on_cran()
+  skip_slow("takes too long to measure coverage")
 
   n <- 1E3
   data <- as.data.frame(replicate(n, 1L, simplify = FALSE))
@@ -87,8 +79,6 @@ test_that("data.frames with many columns survive roundtrip", {
 })
 
 test_that("data.frames with many columns don't cause Java StackOverflows", {
-  skip_on_cran()
-
   version <- Sys.getenv("SPARK_VERSION", unset = "2.2.0")
 
   n <- if (version >= "2.0.0") 500 else 5000
@@ -99,14 +89,14 @@ test_that("data.frames with many columns don't cause Java StackOverflows", {
   expect_true(TRUE, info = "no Java StackOverflow on copy of large dataset")
 })
 
-test_that("'sdf_predict()', 'predict()' return same results", {
+test_that("'ml_predict()', 'predict()' return same results", {
   test_requires("dplyr")
 
   model <- flights_tbl %>%
     na.omit() %>%
     ml_decision_tree(sched_dep_time ~ dep_time)
 
-  predictions <- sdf_predict(model)
+  predictions <- ml_predict(model)
   n1 <- spark_dataframe(predictions) %>% invoke("count")
   n2 <- length(predict(model))
 
@@ -138,31 +128,40 @@ test_that("collect() can retrieve all data types correctly", {
   # https://cwiki.apache.org/confluence/display/Hive/LanguageManual+Types#LanguageManualTypes
   library(dplyr)
 
+  utime <- as.numeric(as.POSIXct("2010-01-01 01:01:10", origin = "1970-01-01", tz = "UTC"))
   sdate <- "from_unixtime(unix_timestamp('01-01-2010' , 'dd-MM-yyyy'))"
   rdate <- as.Date("01-01-2010", "%d-%m-%Y") %>% as.character()
-  rtime <- as.POSIXct(1, origin = "1970-01-01", tz = "UTC") %>% as.character()
+  stime <- paste0("to_utc_timestamp(from_unixtime(", utime, "), 'UTC')")
+  rtime <- "2010-01-01 01:01:10"
+  atime <- as.character(as.POSIXct(utime, origin = "1970-01-01"))
+
+  arrow_compat <- using_arrow() && packageVersion("arrow") < "0.12.0"
 
   hive_type <- tibble::frame_data(
-    ~stype,     ~svalue,       ~rtype,   ~rvalue,
-    "tinyint",       "1",   "integer",       "1",
-    "smallint",      "1",   "integer",       "1",
-    "integer",       "1",   "integer",       "1",
-    "bigint",        "1",   "numeric",       "1",
-    "float",         "1",   "numeric",       "1",
-    "double",        "1",   "numeric",       "1",
-    "decimal",       "1",   "numeric",       "1",
-    "timestamp",     "1",   "POSIXct",     rtime,
-    "date",        sdate,      "Date",     rdate,
-    "string",          1, "character",       "1",
-    "varchar(10)",     1, "character",       "1",
-    "char(10)",        1, "character",       "1",
-    "boolean",    "true",   "logical",    "TRUE"
+    ~stype,      ~svalue,      ~rtype,   ~rvalue,      ~atype,    ~avalue,
+    "tinyint",       "1",   "integer",       "1",       "raw",       "01",
+    "smallint",      "1",   "integer",       "1",   "integer",        "1",
+    "integer",       "1",   "integer",       "1",   "integer",        "1",
+    "bigint",        "1",   "numeric",       "1", "integer64",        "1",
+    "float",         "1",   "numeric",       "1",   "numeric",        "1",
+    "double",        "1",   "numeric",       "1",   "numeric",        "1",
+    "decimal",       "1",   "numeric",       "1",   "numeric",        "1",
+    "timestamp",   stime,   "POSIXct",     rtime,   "POSIXct",      atime,
+    "date",        sdate,      "Date",     rdate,      "Date",      rdate,
+    "string",          1, "character",       "1", "character",        "1",
+    "varchar(10)",     1, "character",       "1", "character",        "1",
+    "char(10)",        1, "character",       "1", "character",        "1",
+    "boolean",    "true",   "logical",    "TRUE",   "logical",     "TRUE",
   )
 
   if (spark_version(sc) < "2.2.0") {
     hive_type <- hive_type %>% filter(stype != "integer")
   }
 
+  if ("arrow" %in% installed.packages() && packageVersion("arrow") < "0.12.0") {
+    hive_type <- hive_type %>% filter(stype != "smallint", stype != "float")
+  }
+
   spark_query <- hive_type %>%
     mutate(
       query = paste0("cast(", svalue, " as ", stype, ") as ", gsub("\\(|\\)", "", stype), "_col")
@@ -177,17 +176,16 @@ test_that("collect() can retrieve all data types correctly", {
 
   expect_equal(
     spark_types,
-    hive_type %>% pull(rtype)
+    hive_type %>% pull(!! if(arrow_compat) "atype" else "rtype")
   )
 
-  spark_results <- DBI::dbGetQuery(sc, spark_query) %>%
-    lapply(as.character)
+  spark_results <- DBI::dbGetQuery(sc, spark_query)
   names(spark_results) <- NULL
-  spark_results <- spark_results %>% unlist()
+  spark_results <- sapply(spark_results, as.character)
 
   expect_equal(
     spark_results,
-    hive_type %>% pull(rvalue)
+    hive_type %>% pull(!! if(arrow_compat) "avalue" else "rvalue")
   )
 })
 
@@ -195,21 +193,29 @@ test_that("collect() can retrieve NULL data types as NAs", {
   library(dplyr)
 
   hive_type <- tibble::frame_data(
-        ~stype,        ~rtype,
-     "tinyint",     "integer",
-    "smallint",     "integer",
-     "integer",     "integer",
-      "bigint",     "numeric",
-       "float",     "numeric",
-      "double",     "numeric",
-     "decimal",     "numeric",
-   "timestamp",     "POSIXct",
-        "date",        "Date",
-      "string",   "character",
- "varchar(10)",   "character",
-    "char(10)",   "character"
+        ~stype,        ~rtype,        ~atype,
+     "tinyint",     "integer",         "raw",
+    "smallint",     "integer",     "integer",
+     "integer",     "integer",     "integer",
+      "bigint",     "numeric",   "integer64",
+       "float",     "numeric",     "numeric",
+      "double",     "numeric",     "numeric",
+     "decimal",     "numeric",     "numeric",
+   "timestamp",     "POSIXct",     "POSIXct",
+        "date",        "Date",        "Date",
+      "string",   "character",   "character",
+ "varchar(10)",   "character",   "character",
+    "char(10)",   "character",   "character",
   )
 
+  if (using_arrow()) {
+    # Disable while tracking fix for ARROW-3794
+    hive_type <- hive_type %>% filter(stype != "tinyint")
+
+    # Disable while tracking fix for ARROW-3795
+    hive_type <- hive_type %>% filter(stype != "bigint")
+  }
+
   if (spark_version(sc) < "2.2.0") {
     hive_type <- hive_type %>% filter(stype != "integer")
   }
@@ -228,7 +234,7 @@ test_that("collect() can retrieve NULL data types as NAs", {
 
   expect_equal(
     spark_types,
-    hive_type %>% pull(rtype)
+    hive_type %>% pull(!! if(using_arrow()) "atype" else "rtype")
   )
 
   spark_results <- DBI::dbGetQuery(sc, spark_query)
@@ -263,6 +269,9 @@ test_that("invoke() can roundtrip collect fields", {
 })
 
 test_that("collect() can retrieve as.POSIXct fields with timezones", {
+  # Disabled in arrow while #1737 is investigated
+  skip_on_arrow()
+
   tz_entries <- list(
     as.POSIXct("2018-01-01", tz = "UTC"),
     as.POSIXct("2018-01-01", tz = "GMT"),
